// ***************************************************************************
// Name:        3D-Wireframe (header file)
// Purpose:     shows wireframe 3d objects
// Copyright:   (C) 2003, 2004, 2005, 2007 Dominik Wagenfuehr <dominik.wagenfuehr@arcor.de>
// ***************************************************************************

/**
This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

#include <wx/wx.h>
#include <math.h>

// **************************
// Vector Class
// **************************

class Vector {					// vector in 3d space                                       
  double xcoord;
  double ycoord;
  double zcoord;

public:
  Vector(double x, double y, double z);               // standard contructor
  Vector(void);                                       // standard contructor
    
  Vector operator=(Vector v);                        // copy constructor
  Vector operator-(void);                              // negation operator
  bool operator==(Vector v);                           // compare operator
  bool operator!=(Vector v);                           // compare operator

  Vector operator+(Vector v);                        // binary addition operator
  Vector operator+=(Vector v);                       // unary addition operator
  Vector operator-(Vector v);                        // binary subtraction operator
  Vector operator-=(Vector v);                       // unary subtraction operator
  Vector operator*=(double r);                         // unary scalar multiplication operator

  friend Vector operator*(double r, Vector v);       // binary scalar multiplication operator
  friend Vector operator*(Vector v, double r);       // binary scalar multiplication operator

  Vector Rotate(double angle, Vector r);             // rotate the vector around vector axis r
  Vector Translate(Vector v);                        // move the vector

  Vector VecProd(Vector v) const;                          // cross product between two vectors
  double ScalProd(Vector v) const;                   // scalar product between two vectors
  double Norm(void) const;                           // norm / length of a vector
  Vector Normalize(double norm=1);                   // normalize a vector to length n
  Vector Orth(Vector r, double norm=1) const;        // calculates a vector that is orthogonal with norm
  double Angle(Vector v) const;                      // calculate the angle between two vectors

  double x(void) const;                              // returns x-coord
  double y(void) const;                              // returns y-coord
  double z(void) const;                              // returns z-coord

  void Print(wxString string="") const;                      // prints all coordiantes on screen
};

// **************************
// VectorList Class (Definition)
// **************************

struct VectorList {
  VectorList *last;
  Vector element;
  VectorList *next;
};

// **************************
// ArrayOfVectors Class
// **************************

class ArrayOfVectors {
  VectorList *anchor;                         // anchor for list with vectors
  size_t size;                                // size of the array

  size_t Count(void);                         // count the elements explicitly
  
public:
  ArrayOfVectors(void);                       // standard contructor
  ArrayOfVectors(Vector v);                   // standard contructor
//  ~ArrayOfVectors();                          // destructor

  Vector& operator[](size_t index);           // get size of the array
  void Add(Vector v);                         // add a vector to the list
  void Delete(size_t index);                  // delete vector at position index
  void Empty(void);                           // clears list
  VectorList *GetAnchor(void) const;          // get anchor of the array
  size_t GetCount(void) const;                // get size of the array
  Vector& GetItem(size_t index);         // get item at position index
  bool IsEmpty(void) const;                   // check if list is empty
  void SetItem(size_t index, Vector v);       // set item at position index to vector
};

// **************************
// Vertex Class
// **************************

class Vertex {					// Vertex in 3d space                                       
  Vector vertex;                                                 
  ArrayOfVectors edge;         // edges to other points

public:
  Vertex(double x=0, double y=0, double z=0);          // standard contructor
  Vertex(Vector v);                                    // standard contructor
//  ~Vertex();                                           // destructor

  void AddEdge(Vector v);                             // add a edge to the list
  void DeleteEdge(size_t index);                      // delete vector at position index
  void EmptyEdges(void);                              // clears list
  VectorList *GetEdgeAnchor(void) const;              // get anchor of the array
  size_t GetEdgeCount(void) const;                    // get size of the array
  Vector& GetEdgeItem(size_t index);                 // get item at position index
  bool IsEdgeEmpty(void) const;                      // check if list is empty
  void SetEdgeItem(size_t index, Vector v);          // set item at position index to vector

  Vertex Rotate(double angle, Vector r);             // rotate the vector and all edges around vector axis r
  Vertex Translate(Vector v);                        // move the vector and edges

  Vector GetVector(void) const;                         // returns vector
  double x(void) const;                                // returns x-coord
  double y(void) const;                                // returns y-coord
  double z(void) const;                                // returns z-coord
};

// **************************
// VertexList Class (Definition)
// **************************

struct VertexList {
  VertexList *last;
  Vertex element;
  VertexList *next;
};

// **************************
// ArrayOfVertices Class
// **************************

class ArrayOfVertices {
  VertexList *anchor;                         // anchor for list with vertices       
  size_t size;                                // size of the array

  size_t Count(void);                         // count the elements explicitly
  
public:
  ArrayOfVertices(void);                       // standard contructor
  ArrayOfVertices(Vertex v);                   // standard contructor
//  ~ArrayOfVertices();                          // destructor

  Vertex& operator[](size_t index);           // get size of the array
  void Add(Vertex v);                         // add a vertex to the list
  void Delete(size_t index);                  // delete vertex at position index
  void Empty(void);                           // clear list
  VertexList *GetAnchor(void) const;          // get anchor of the array
  size_t GetCount(void) const;                // get size of the array
  Vertex& GetItem(size_t index);              // get item at position index
  bool IsEmpty(void) const;                   // check if list is empty
  void SetItem(size_t index, Vertex v);       // set item at position index to vertex
};

// **************************
// Model Class
// **************************

class Model {					       // 3D-Model
  ArrayOfVertices vertices;           // vertices of the Model

public:
  Model(ArrayOfVertices vert=ArrayOfVertices());           // standard contructor
//  ~Model();                                                // destructor

  void AddVertex(Vertex v);                         // add a vertex to the list
  void DeleteVertex(size_t index);                  // delete vertex at position index
  void EmptyVertices(void);                           // clear list
  VertexList *GetVertexAnchor(void) const;          // get anchor of the array
  size_t GetVertexCount(void) const;                // get size of the array
  Vertex& GetVertexItem(size_t index);              // get item at position index
  bool IsVertexEmpty(void) const;                   // check if list is empty
  void SetVertexItem(size_t index, Vertex v);       // set item at position index to vertex

  Model Rotate(double angle, Vector r);                    // rotate the Model around vector axis r
  Model Translate(Vector v);                               // move the Model

  Model CreateQuader(double x, double y, double z);    // creates quader
  Model CreateQuader(Vector v);                        // creates quader
};

// **************************
// ModelList Class (Definition)
// **************************

struct ModelList {
  ModelList *last;
  Model element;
  ModelList *next;
};

// **************************
// ArrayOfModels Class
// **************************

class ArrayOfModels {
  ModelList *anchor;                         // anchor for list with 3D models       
  size_t size;                                // size of the array

  size_t Count(void);                         // count the elements explicitly
  
public:
  ArrayOfModels(void);                        // standard contructor
  ArrayOfModels(Model v);                     // standard contructor
//  ~ArrayOfModels();                          // destructor

  Model& operator[](size_t index);           // get size of the array
  void Add(Model v);                         // add a vertex to the list
  void Delete(size_t index);                  // delete vertex at position index
  void Empty(void);                           // clear list
  ModelList *GetAnchor(void) const;          // get anchor of the array
  size_t GetCount(void) const;                // get size of the array
  Model& GetItem(size_t index);              // get item at position index
  bool IsEmpty(void) const;                   // check if list is empty
  void SetItem(size_t index, Model v);       // set item at position index to vertex
};

// **************************
// Frame Class
// **************************

class ImageFrame : public wxFrame {             // main window class    
  int maxx, maxy;                       // maximum horizontal and vertical screen size
  wxPoint mousepos;                     // mouse position in frame
  
  Vector standpoint, viewpoint;         // position in space and direction you are looking to
  Vector normvector1, normvector2;      // two orthogonal vectors to the view direction

  ArrayOfModels scene;                   // 3D models in the Scene

  void DrawLine(wxPaintDC *dc, Vector v, Vector w);		// draw line between two points
  void DrawScene(wxPaintDC *dc);                     // draw current scene on screen
  void DrawPoint(wxPaintDC *dc, Vector v, wxCoord radius=0);           // draw a single point in space
  void DrawVector(wxPaintDC *dc, Vector v);          // draw the vector
  void DrawVertex(wxPaintDC *dc, Vertex v);          // draw a vertex with all edges
  void DrawModel(wxPaintDC *dc, Model m);           // draw the whole model
  
public :
  ImageFrame ( const wxString& title, const wxPoint& pos, const wxSize& size, long style=wxDEFAULT_FRAME_STYLE ); // constructor

  void OnPaint (wxPaintEvent& event);                 // paint wireframe-model
  void Look (wxMouseEvent & event);                   // mouse is moved

  void SetViewPoint(Vector w);                        // set new ViewPoint
  void SetStandPoint(Vector w);                        // set new StandPoint
  bool IsVisible(Vector w, wxRealPoint *p=NULL) const;              // returns the point coordinates if the vector is visible in the frame
  
  void AddModel(Model v);                           // add a model to the list
  void DeleteModel(size_t index);                   // delete model at position index
  void EmptyModels(void);                           // clear list
  ModelList *GetModelAnchor(void) const;          // get anchor of the array
  size_t GetModelCount(void) const;                 // get size of the array
  Model& GetModelItem(size_t index);                // get item at position index
  bool IsModelEmpty(void) const;                   // check if list is empty
  void SetModelItem(size_t index, Model v);         // set item at position index to vertex

  Vector Ex;                            // vector (1,0,0)
  Vector Ey;                            // vector (0,1,0)
  Vector Ez;                            // vector (0,0,1)

protected:
  DECLARE_EVENT_TABLE()
};

